package com.lanren.huhu.partner.schedule;

import com.baomidou.mybatisplus.autoconfigure.MybatisPlusProperties;
import com.lanren.huhu.partner.constants.Constants;
import com.lanren.huhu.partner.domain.AgentExpandDaily;
import com.lanren.huhu.partner.domain.AgentExpandDetail;
import com.lanren.huhu.partner.domain.PartnerInviteRelation;
import com.lanren.huhu.partner.domain.UserAgent;
import com.lanren.huhu.partner.model.User;
import com.lanren.huhu.partner.service.AgentExpandDailyService;
import com.lanren.huhu.partner.service.AgentExpandDetailService;
import com.lanren.huhu.partner.service.PartnerInviteRelationService;
import com.lanren.huhu.partner.service.UserService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.Date;
import java.util.List;

import static com.lanren.huhu.partner.constants.Constants.PARENT_COLUMN_NAME_CENGJI;

/**
 * @author chen
 * @title: AgentDailyExpandTask
 * @projectName partner
 * @description: 代理商每日拓展用户数,  按用户的代理商层次查找归属
 * @package com.lanren.huhu.partner.schedule
 * @date 2019-07-08 15:50
 */
@Component
public class AgentDailyExpandTask {
    private static Logger logger = LoggerFactory.getLogger(AgentDailyExpandTask.class);

    @Autowired
    PartnerInviteRelationService partnerInviteRelationService;
    @Autowired
    UserService userService;
    @Autowired
    AgentExpandDetailService agentExpandDetailService;
    @Autowired
    AgentExpandDailyService agentExpandDailyService;
    @Autowired
    MybatisPlusProperties mybatisPlusProperties;

//    @Scheduled(cron="0 5 */2 * * *")
    @Scheduled(fixedDelay = 10000L)
    public void runScheduledTask() {
        logger.info("MapperLocations is: {}", mybatisPlusProperties.getMapperLocations());
//        logger.info("run AgentDailyExpandTask");
//        runSummary();
    }

    @Async
    public void runSummary() {
        SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd");
        int now = (int)(System.currentTimeMillis() / 1000L);
        Date dat = new Date(System.currentTimeMillis() - 10 * 60 * 1000L);
        String datStr = df.format(dat);
        datStr = "2019-07-06";
        List<PartnerInviteRelation> partnerInviteRelationList = partnerInviteRelationService.getPartnerInviteRelationListByRegDay(datStr);
        for (PartnerInviteRelation partnerInviteRelation : partnerInviteRelationList) {
            List<UserAgent> userAgentList = userService.getUserAgentChildListByUserId(partnerInviteRelation.getUserId(), PARENT_COLUMN_NAME_CENGJI);
            for (UserAgent userAgent : userAgentList) {
                AgentExpandDetail detail = new AgentExpandDetail();
                detail.setDat(dat);
                detail.setAgentId(userAgent.getAgentId());
                detail.setUserId(userAgent.getUserId());
                detail.setExpandUserId(partnerInviteRelation.getUserId());
                detail.setExpandUserRegTime(new Date(partnerInviteRelation.getRegiterTime() * 1000L));
                detail.setCreatedAt(now);
                detail.setUpdatedAt(now);
                agentExpandDetailService.insertOrUpdate(detail);
            }
        }
        if (partnerInviteRelationList.size() > 0) {
            agentExpandDailyService.updateByDatByDetail(datStr);
        }
    }
}
